/*
 * $Log: DTWWRAPImpl.cpp,v $
 * Revision 1.3  2002/03/06 22:11:24  timo
 * A couple of bug fixes
 *
 * Revision 1.1  2002/03/03 18:22:49  jylipekk
 * wrapper code added
 *
 */

/*
#define WRAPDEBUG 0
*/

#include "org_dtrend_DTWWRAP.h"
#include "dtwwrapimpl.h"
#include <stdlib.h>

#ifdef WRAPDEBUG
#include <iostream>
#include <fstream>
using namespace std;
#endif



typedef int (* DTADDTEMPLATE)(float* tSeq, int nSeq);
typedef int (* DTDELTEMPLATE)(int tIndex);

typedef DTWAllResultsElement* (* DTCOMPAREALL)(float *oSeq, 
							int nSeq, 
							DTWParameters params, 
							int* numresults);


typedef DTWOneResult* (* DTCOMPAREONE)(float* oSeq,
						int nSeq,
						DTWParameters params,
						int tIndex);

typedef float* (* DTGETTEMPLATE)(int tIndex, int* nSeq);
typedef	int (* DTFREERESOURCES)(void);



/* global data structures */

DTWAllResultsElement* dtw_all_results_array=NULL; 
DTWParameters *dtwparams=NULL;
DTWOneResult* dtw_one_result=NULL;


int dtw_all_results_array_len = 0;
int last_run_tindex=-2;

/* freeing of global data structures is handled both in dll exit and after consequent
 * calls to this dll
 */



/* function for adding a template into the dtw library's data structures
 *
 * jf_array - a Java/JNI float array
 */
JNIEXPORT jint JNICALL Java_org_dtrend_DTWWRAP_addTemplate (JNIEnv *env, jobject obj, jfloatArray jf_array) {
  
	int tmp_tIndex = 0;
	jsize nSeq = (jsize) env->GetArrayLength(jf_array);
	int i, sum = 0;
	jfloat *tSeq = (jfloat *) env->GetFloatArrayElements(jf_array, 0);
	
	i = DTAddTemplate((float *) tSeq, (int) nSeq);
	tmp_tIndex = (jint) i;

	env->ReleaseFloatArrayElements(jf_array,tSeq,0);

	#ifdef WRAPDEBUG	
	ofstream debug("wrapdebug.txt",ios_base::out|ios_base::app);
	debug << "Added template " << i << endl;
	debug.close();
	#endif

	return tmp_tIndex;      
}

/* function for fetching a template from the dtw library's data structures
 *
 * tIndex - a Java/JNI integer value
 */
JNIEXPORT jfloatArray JNICALL Java_org_dtrend_DTWWRAP_getTemplate(JNIEnv *env, jobject obj, jint tIndex) {
	
	int len = 0;
		  
	float *retseq = DTGetTemplate((int) tIndex, &len);
	jfloatArray jfarray = (jfloatArray) env->NewFloatArray((jsize) len);
	env->SetFloatArrayRegion(jfarray, 0, (jsize) len, retseq);
	DTFreeResources();
	return jfarray;
}

/* function for deleting a specified tempate from the dtw library's structures
 *
 * tIndex - a Java/JNI integer value
 */
JNIEXPORT jboolean JNICALL Java_org_dtrend_DTWWRAP_delTemplate(JNIEnv *env, jobject obj, jint tIndex) {
	int retval = DTDelTemplate((int) tIndex);
	jboolean retjb;
	if(retval == 1) {
		retjb = JNI_FALSE;
	} else {
		retjb = JNI_TRUE;
	}
	return retjb;
}

/* function for running the dtw algorithm against the given observation time-series and
 * all the templates currently stored in the dtw library data structures
 *
 * jfarray - a Java/JNI float array (the observation time-series)
 * p1 - a Java/JNI integer value (dmetrics)
 * p2 - a Java/JNI integer value (window width)
 * p3 - a Java/JNI integer value (sconstraint)
 * p4 - a Java/JNI integer value (seoffset)
 */
JNIEXPORT jboolean JNICALL Java_org_dtrend_DTWWRAP_CompareAll(JNIEnv *env, jobject obj, jfloatArray jf_array,
		jint p1, jint p2, jint p3, jint p4) {

	jsize nSeq = (jsize) env->GetArrayLength(jf_array);
	int numresults = 0;
	jfloat *oSeq = (jfloat *) env->GetFloatArrayElements(jf_array, 0);

	dtwparams = (DTWParameters *) calloc(1, sizeof(DTWParameters));
	
	/* set the algorithm parameters in place */
	(*dtwparams).dmetrics = (int) p1;
	(*dtwparams).wwidth = (int) p2;
	(*dtwparams).sconstraint = (int) p3;
	(*dtwparams).seoffset = (int) p4;

	/* now do the function call and check the results */
	dtw_all_results_array = DTCompareAll(oSeq, (int) nSeq, *dtwparams, &numresults);
	
	env->ReleaseFloatArrayElements(jf_array,oSeq,0);

	if(dtw_all_results_array != NULL) {
		dtw_all_results_array_len = numresults;
		return JNI_TRUE;
	} else {
		return JNI_FALSE;
	}
	/* now we wait for the atomic calls:
	 * 1. getResult_1sortedIndex()
	 * 2. getResult_1PathX()
	 * 3. getResult_1PathY()
	 * 4. getResult_1dtwDistance()
	 * 5. freeDataStructures()
	 */	
	
}
	
JNIEXPORT jboolean JNICALL Java_org_dtrend_DTWWRAP_CompareOne(JNIEnv *env, jobject obj, jint tIndex, jfloatArray jf_array,
		jint p1, jint p2, jint p3, jint p4) {

	jsize nSeq = (jsize) env->GetArrayLength(jf_array);
	
	dtwparams = (DTWParameters *) calloc(1, sizeof(DTWParameters));
	
	jfloat *oSeq = (jfloat *) env->GetFloatArrayElements(jf_array, 0);
	/* set the algorithm parameters in place */
	(*dtwparams).dmetrics = (int) p1;
	(*dtwparams).wwidth = (int) p2;
	(*dtwparams).sconstraint = (int) p3;
	(*dtwparams).seoffset = (int) p4;

	/* now do the function call and check the results */
	if(dtw_one_result =  DTCompareOne(oSeq, (int) nSeq, *dtwparams, tIndex)) {
		
		env->ReleaseFloatArrayElements(jf_array,oSeq,0);
		
		#ifdef WRAPDEBUG	
		ofstream debug("wrapdebug.txt",ios_base::out|ios_base::app);
		debug << "Compared to template " << tIndex << endl;
		debug.close();
		#endif
		
		last_run_tindex=tIndex;

		return JNI_TRUE;
	} else {
		return JNI_FALSE;
	}
	
	/* now we wait for the atomic calls:
	 * 1. getResult_1sortedIndex()
	 * 2. getResult_1PathX()
	 * 3. getResult_1PathY()
	 * 4. getResult_1dtwDistance()
	 * 5. freeDataStructures()
	 */	
}

JNIEXPORT jintArray JNICALL Java_org_dtrend_DTWWRAP_getResult_1sortedIndex(JNIEnv *env, jobject obj) {
	
	jintArray jiarray = (jintArray) env->NewIntArray((jsize) dtw_all_results_array_len);
	

	if (dtw_all_results_array_len > 0) {
		int i = 0;
		int *sorted_index = new int[dtw_all_results_array_len];
		
		for(i = 0; i<dtw_all_results_array_len; i++) {
			sorted_index[i] = dtw_all_results_array[i].tIndex;
		}

		env->SetIntArrayRegion(jiarray, 0, (jsize) dtw_all_results_array_len, (jint *) sorted_index);	
	} 
	return jiarray;
}

JNIEXPORT jfloat JNICALL Java_org_dtrend_DTWWRAP_getResult_1dtwDistance(JNIEnv *env, jobject obj, jint tIndex) {
	jfloat jf_dtw_distance = 0;

	if (tIndex>=0) 
		jf_dtw_distance = (jfloat) dtw_all_results_array[tIndex].dtwDistance;
	else
		jf_dtw_distance = (jfloat) dtw_one_result->dtwDistance;

	return jf_dtw_distance;
}


JNIEXPORT jintArray JNICALL Java_org_dtrend_DTWWRAP_getResult_1PathX(JNIEnv *env, jobject obj) {
	int i,pl;
	int* temparray;

	pl=dtw_one_result->pathLen;

	jintArray jiarray = (jintArray) env->NewIntArray((jsize) pl);
	
	temparray=new int[pl];
	for(i=0;i<pl;i++)
		temparray[i]=dtw_one_result->warpingPath[i][0];



	env->SetIntArrayRegion(jiarray, 0, (jsize)pl, (jint *) temparray);
	return jiarray;
}

JNIEXPORT jintArray JNICALL Java_org_dtrend_DTWWRAP_getResult_1PathY(JNIEnv *env, jobject obj) {
	int i,pl;
	int* temparray;

	pl=dtw_one_result->pathLen;

	jintArray jiarray = (jintArray) env->NewIntArray((jsize) pl);
	
	temparray=new int[pl];
	for(i=0;i<pl;i++)
		temparray[i]=dtw_one_result->warpingPath[i][1];

	env->SetIntArrayRegion(jiarray, 0, (jsize)pl, (jint *) temparray);
	return jiarray;
}

JNIEXPORT jboolean JNICALL Java_org_dtrend_DTWWRAP_freeDataStructures(JNIEnv *env, jobject obj) {
	
	DTFreeResources();

	return JNI_TRUE;
}


