/**
 * $Log: DTWDLLApi.java,v $
 * Revision 1.5  2002/03/18 09:11:28  timo
 * Added a confirm dialog when trying to overwrite files and changed the default
 * directory of file dialogs to working directory
 *
 * Revision 1.4  2002/03/10 15:42:49  timo
 * Fixed a but with getting results of DTCompareALL
 *
 * Revision 1.3  2002/03/04 11:21:59  timo
 * A few (undocumented ;) changes to the code
 *
 * Revision 1.2  2002/03/02 15:18:43  jylipekk
 *
 * DTWDLLApi.CompareOne() now returns an array of DTWAllResultsElement objects.
 *
 * Revision 1.1  2002/02/21 14:52:44  jylipekk
 * DTWDLLApi almost finished.
 *
 *
 */


package org.dtrend;

import java.lang.*;
/** This class provides a Java interface for class org.dtrend.DTWData to the DTWLIB.DLL
 * @author Juha Ylipekkala
 * @version 0.1
 */
public class DTWDLLApi {
	

	private DTWWRAP dtwlib = null;

	public DTWDLLApi() {
		
		this.dtwlib = new DTWWRAP();
		
	}
	/** Adds a template time-series to the dll's data structures
	 * @param vec The time-series in floating point values
	 * @return Integer number representing the index inside the DLL.
	 */
	public int addTemplate(float [] vec) {
		
		/** the library call returns an integer index number */
		return this.dtwlib.addTemplate(vec);
	
	} /** addTemplate() ends */

	/** Returns a specified template from the DLL
	 * @param tIndex An integer number representing an index in the DLL.
	 * @return A floating point value array containing the template time-series values
	 */
	public float[] getTemplate(int tIndex) {

		float[] tmp_template = this.dtwlib.getTemplate(tIndex);
		return tmp_template;

	} /** getTemplate() ends */

	/** Deletes a specified template from the DLL data structures
	 * @param tIndex An integer number representing an index in the DLL
	 * @return boolean value true if deletion succeeded, otherwise false
	 */
	public boolean delTemplate(int tIndex) {
		
		return this.dtwlib.delTemplate(tIndex);

	} /** delTemplate() ends */

	/** Compares all templates agains the given time-series and algorithm parameters
	 * @param oSeq An array of floating point values, containing the observation time-series
	 * @param param An org.dtrend.DTWParameters object which contains the parameters wanted
	 * @return A sorted array of DTWOneResult objects, best match at index 0.
	 */
	public DTWAllResultsElement[] CompareAll(float [] oSeq, DTWParameters param) {
	    DTWAllResultsElement[] retarray;

		java.util.Vector tmp_allresults_array = new java.util.Vector();
		if( this.dtwlib.CompareAll( oSeq, 
									param.getDmetrics(), 
									param.getWwidth(), 
									param.getSconstraint(), 
									param.getSeoffset() ) == true) {
									
			int[] result_index_array = this.dtwlib.getResult_sortedIndex();
			
			for(int i = 0; i < result_index_array.length; i++) {
			
				DTWAllResultsElement tmp_allresultselement = new DTWAllResultsElement();
			
				tmp_allresultselement.setTIndex(result_index_array[i]);
				
				tmp_allresultselement.setDtwDistance(this.dtwlib.getResult_dtwDistance(i)); //HUOM! EI result_index_array[i]
				
				/** append the new DTWOneResult to the Vector */
				tmp_allresults_array.add(tmp_allresultselement);
			}
			/** succesfully created a vector of results, now return the array to caller
			 *  after freeing the data structures in the wrapper library
			 */
			this.dtwlib.freeDataStructures();
			retarray=new DTWAllResultsElement[tmp_allresults_array.size()];
			tmp_allresults_array.toArray(retarray);
			return retarray;
		} else {
			return null;
		}
		
	} /** CompareAll() ends */

	/** Compares one specified template against the given observation time-series with specified parameters.
	 * @param oSeq An array of floating point values, containing the observation time-series
	 * @param param An org.dtrend.DTWParameters object which contains the parameters wanted
	 * @param tIndex An integer value representing exactly one template in the DLL
	 * @return An org.dtrend.DTWOneResult object with results from the comparison
	 */
	public DTWOneResult CompareOne(float [] oSeq, DTWParameters param, int tIndex) {
		
	    if( this.dtwlib.CompareOne( tIndex, 
									oSeq, 
									param.getDmetrics(), 
									param.getWwidth(), 
									param.getSconstraint(), 
									param.getSeoffset() ) == true) {
									
			DTWOneResult tmp_oneresult = new DTWOneResult();
			
			tmp_oneresult.setTIndex(tIndex);
				
			tmp_oneresult.setDtwDistance(this.dtwlib.getResult_dtwDistance(-1));
				
			int [] tmp_wpath_x = this.dtwlib.getResult_PathX();
			int [] tmp_wpath_y = this.dtwlib.getResult_PathY();
			int [][] tmp_wpath = new int[tmp_wpath_x.length][];
			
			for(int i=0;i<tmp_wpath_x.length;i++) {
			    tmp_wpath[i]=new int[2];
			    tmp_wpath[i][0]=tmp_wpath_x[i];
			    tmp_wpath[i][1]=tmp_wpath_y[i];
			}
	
			tmp_oneresult.setWarpingPath(tmp_wpath);
			
			/** succesfully compared and created the result object, now return it to caller
			 *  aftger clearing the data structures in the wrapper library
			 */	
			this.dtwlib.freeDataStructures();
			return tmp_oneresult;
			
		} else {
			return null;
		}
		
		

	} /** CompareOne() ends */

}


	
	

