/*
DTWMain.java -- DTREND PROJECT
Author: Tero Rontti
Version: 0.1
Revision date: 23.2.2002
Creation date: 10.2.2002


DTREND PROJECT
Project manager: Juha Ylipekkala
Project group: Timo Ahonen, Tero Rontti, Juha Ylipekkala

-------------------------------------------------------------------------------
REVISION HISTORY

Version 0.1
Initial working version
Author: Tero Rontti
Date: 23.2.2002
Known bugs/problems:
- GUI not fully implemented. TODO:
 * Pathplotter showing warping window
 * Pathplotter warping path lenght
 * Check algorithm restrictions on algorithm run
 * Confirmations on deletions and overwriting files
*/

package org.dtrend;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** This class contains the main layout and functionalities of the GUI for
 * DTREND tool.
 * @author Tero Rontti
 * @version 0.1
 */
public class DTWMain extends JFrame {
    /** File IO object */
    DTWFileIO FileIO = new DTWFileIO();

    /** Data object */
    DTWData Data = new DTWData();

    /** Layout for the frame */
    GridBagLayout gridBagLayout2 = new GridBagLayout();

    /** File menu objects */
    JMenuBar menuBar1 = new JMenuBar();
    JMenu menuFile = new JMenu();
    JMenuItem menuFileExit = new JMenuItem();
    JMenuItem jMenuItem1 = new JMenuItem();
    JMenuItem jMenuItem2 = new JMenuItem();
    JMenuItem jMenuItem3 = new JMenuItem();
    JMenuItem jMenuItem4 = new JMenuItem();
    JMenuItem jMenuItem5 = new JMenuItem();

    /** Action button panel */
    JPanel buttonPanel = new JPanel();
    GridLayout gridLayout1 = new GridLayout();
    JButton compareOneButton = new JButton("Compare");
    JButton compareAllButton = new JButton("Compare All");
    JButton deleteAllButton = new JButton("Delete all");
    JButton deleteOneButton = new JButton("Delete");
    JButton logButton = new JButton("Log");

    /** VTT Logo panel */
    JPanel logoPanel = new JPanel();
    JLabel logo = new JLabel();

    /** Template list */
    JScrollPane templatePanel = new JScrollPane();
    JList jList1 = new JList();

    /** Parameter panel */
    JPanel parameterPanel = new JPanel();
    ButtonGroup buttonGroup1 = new ButtonGroup();
    GridLayout gridLayout2 = new GridLayout(4,1);
    DTWParameters parameters = new DTWParameters();

    /** Lower parameter panel */
    JPanel jPanel1 = new JPanel();
    JCheckBox jCheckBox2 = new JCheckBox();
    JCheckBox jCheckBox1 = new JCheckBox();
    JTextField jTextField1 = new JTextField();
    String prevValue1 = "0";
    JTextField jTextField2 = new JTextField();
    String prevValue2 = "0";
    GridLayout gridLayout5 = new GridLayout();

    /** Upper parameter panel */
    JPanel jPanel2 = new JPanel();
    JRadioButton jRadioButton1 = new JRadioButton();
    JRadioButton jRadioButton2 = new JRadioButton();
    GridLayout gridLayout4 = new GridLayout();

    /** Plotter panel */
    JPanel plotterPanel = new JPanel();
    GridBagLayout gridBagLayout6 = new GridBagLayout();
    DTWHorizontalPlotter ObsPlotter = new DTWHorizontalPlotter(470, 100);
    DTWVerticalPlotter TempPlotter = new DTWVerticalPlotter(120, 350);
    DTWPathPlotter PathPlotter = new DTWPathPlotter( 470, 350 );

    /** Dialog components */
	/*    JDialog err;
    JLabel msg;
    JButton but;
    JPanel pan;*/

    /** Create the frame */
    public DTWMain() {
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try  {
			jbInit();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
    }

    /** Component initialization
     * @throws Exception Any exception caught is forwarded to parent.
     */
    private void jbInit() throws Exception  {
	/** Set frame layout */
		this.getContentPane().setLayout(gridBagLayout2);
		gridLayout1.setColumns(2);
		gridLayout1.setRows(3);
		
		/** Add text fields and checkboxes to lower JPanel */
		jPanel2.setLayout(gridLayout4);
		gridLayout4.setColumns(2);
		gridLayout4.setRows(2);
		
		jCheckBox2.setText("Starting/ending point offset");
		jCheckBox2.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					jCheckBox2_ItemStateChange(e);
				}
			});
		parameters.setSeoffset(0);
		
		jCheckBox1.setText("Warping window width");
		jCheckBox1.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					jCheckBox1_ItemStateChange(e);
				}
			});
		parameters.setWwidth(DTWParameters.NoWarpingWindow);
		parameters.setSconstraint( 0 );
		
		jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jMenuItem2_actionPerformed(e);
				}
			});
		jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jMenuItem3_actionPerformed(e);
				}
			});
		jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jMenuItem4_actionPerformed(e);
				}
			});
		jMenuItem5.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jMenuItem5_actionPerformed(e);
				}
			});
		jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jMenuItem1_actionPerformed(e);
				}
			});
		
		
		jTextField1.disable();
		jTextField1.addFocusListener(new FocusListener(){
				public void focusLost(FocusEvent e){
					updatePar1();
				}
				public void focusGained(FocusEvent e){
				}
			});
		jTextField2.disable();
		jTextField2.addFocusListener(new FocusListener(){
				public void focusLost(FocusEvent e){
					updatePar2();
				}
				public void focusGained(FocusEvent e){
				}
			});
		
		jRadioButton1.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					jRadioButton1_itemStateChanged(e);
				}
			});
		jRadioButton2.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					jRadioButton2_itemStateChanged(e);
				}
			});
		jPanel2.add(jCheckBox1, null);
		jPanel2.add(jTextField1, null);
		jTextField1.setText("0");
		jPanel2.add(jCheckBox2, null);
		jPanel2.add(jTextField2, null);
		jTextField2.setText("0");
		
		/** Add radio buttons to the upper panel */
		jPanel1.setLayout(gridLayout5);
		gridLayout5.setColumns(1);
		gridLayout5.setRows(2);
		jRadioButton1.setText("Euclidian distance metrics");
		jRadioButton1.setSelected(true);
		parameters.setDmetrics(DTWParameters.EuclidianDistanceMetrics);
		jRadioButton2.setText("Derivative distance metrics");
		buttonGroup1.add(jRadioButton1);
		buttonGroup1.add(jRadioButton2);
		jPanel1.add(jRadioButton1, null);
		jPanel1.add(jRadioButton2, null);
		
		/** Add parameter JPanels to parameterPanel */
		gridLayout2.setRows(2);
		parameterPanel.setLayout(gridLayout2);
		parameterPanel.add(jPanel1, null);
		parameterPanel.add(jPanel2, null);
		
		/** Button action listeners. */
		compareOneButton.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					compareOneButton_mouseClicked(e);
				}
			});
		compareAllButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					compareAllButton_actionPerformed(e);
				}
			});
		deleteOneButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					deleteOneButton_actionPerformed(e);
				}
			});
		deleteAllButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					deleteAllButton_actionPerformed(e);
				}
			});
		logButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					logButton_actionPerformed(e);
				}
			});
        logButton.disable();
        logButton.setForeground(java.awt.Color.gray);
		
		/** Set layout for button panel and add the buttons */
		buttonPanel.setLayout(gridLayout1);
		buttonPanel.add(compareOneButton, null);
		buttonPanel.add(compareAllButton, null);
		buttonPanel.add(deleteOneButton, null);
		buttonPanel.add(deleteAllButton, null);
		buttonPanel.add(logButton, null);
		this.getContentPane().add(logoPanel,         new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
																			,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 20, 0), 262, 42));
		
		/** Set up logo panel */
		java.net.URL url=this.getClass().getResource("vtt.gif");

		ImageIcon icon = new ImageIcon(url);
		logo = new JLabel(icon);
		logoPanel.add(logo);
		this.getContentPane().add(templatePanel,  new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
																		 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(-21, 0, 21, 0), 951, 330));
		templatePanel.getViewport().add(jList1, null);
		
		/** Add different panels to the frame */
		this.getContentPane().add(parameterPanel, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
																		 ,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(-8, 0, 28, 0), 131, -2));
		this.getContentPane().add(plotterPanel,   new GridBagConstraints(1, 0, 1, 2, 0.0, 0.0
																		 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(1, 0, 19, 0), 8, 31));
		this.getContentPane().add(buttonPanel,    new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
																		 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(-24, 0, 42, 0), 302, 40));
		
		/** Add plotters to Plotterpanel */
		plotterPanel.setLayout(gridBagLayout6);
		plotterPanel.add(TempPlotter, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
															 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		plotterPanel.add(ObsPlotter,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
															 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		plotterPanel.add(PathPlotter, new GridBagConstraints(1, 0, 2, 1, 0.0, 0.0
															 ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		
		jList1.addMouseListener( new MouseListener(){
				public void mouseClicked(MouseEvent e) {
					ListSelection(e);
				}
				public void mouseExited(MouseEvent e){ }
				public void mousePressed(MouseEvent e){ }
				public void mouseReleased(MouseEvent e){ }
				public void mouseEntered(MouseEvent e){ }
			});
		jList1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		/** Set frame title. */
		this.setSize(new Dimension(850, 692));
		this.setResizable(false);
		this.setTitle("Dynamic Time Warping Tool");

		/** Set menu texts. */
		jMenuItem1.setText("Load data set");
		jMenuItem2.setText("Load observation");
		jMenuItem3.setText("Load templates");
		jMenuItem4.setText("Save observation");
		jMenuItem5.setText("Save templates");
		
		menuFile.setText("File");
		menuFileExit.setText("Exit");
		menuFileExit.addActionListener(new ActionListener()  {
				
				public void actionPerformed(ActionEvent e) {
					fileExit_actionPerformed(e);
				}
			});
		
		/** Add menu items to menu */
		menuFile.add(jMenuItem1);
		menuFile.addSeparator();
		menuFile.add(jMenuItem2);
		menuFile.add(jMenuItem3);
		menuFile.addSeparator();
		menuFile.add(jMenuItem4);
		menuFile.add(jMenuItem5);
		menuFile.addSeparator();
		menuFile.add(menuFileExit);
		menuBar1.add(menuFile);
		this.setJMenuBar(menuBar1);
		
		/*		err = new JDialog( this, true );
		msg = new JLabel();
		but = new JButton( "OK" );
		pan = new JPanel( );
		err.setSize( 200, 100 );
		err.getContentPane().add(pan);
		pan.setLayout( new FlowLayout() );
		pan.add(msg);
		err.getContentPane().add(but, BorderLayout.SOUTH);
		err.dispose();
		
		but.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					err.dispose();
				}
				});*/
		
    }
	
    /** File | Exit action performed */
    public void fileExit_actionPerformed(ActionEvent e) {
		System.exit(0);
    }

    /** Overridden so we can exit on System Close */
    protected void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if(e.getID() == WindowEvent.WINDOW_CLOSING) {
			fileExit_actionPerformed(null);
		}
    }
	
    /** CompareOne button clicked */
    void compareOneButton_mouseClicked(MouseEvent e) {
        int row;
        row = jList1.getSelectedIndex();
        if( row != -1 ){
			int ol = (Data.getObservation()).getValues().length;
			int tl = (Data.getTemplate(row)).getValues().length;
			int ww = Data.getWindowWidth();
			if(java.lang.Math.abs(tl-ol) <= ww){

				Data.DTWCompareOne(row);
				ShowResults();
				logButton.enable();
				logButton.setForeground(java.awt.Color.black);
			}
			else{
				JOptionPane.showMessageDialog( this, new String("Warping window too strict for the template"));
			}
        }
    } /** compareOneButton_mouseClicked() ends */

    /** CompareAll button clicked */
    void compareAllButton_actionPerformed(ActionEvent e) {
        boolean ok;
	DTWDataVector[] templates=Data.getTemplates();
	int i=0;
	int ol = (Data.getObservation()).getValues().length;
	int tl;
	int ww = Data.getWindowWidth();
	
	ok= (templates.length>0);

	while((i<templates.length)&&(ok)) {
	   tl = (Data.getTemplate(i)).getValues().length;
	   if(java.lang.Math.abs(tl-ol) > ww) {
	       JOptionPane.showMessageDialog( this, new String("Warping window too strict for template "+templates[i].getName()));
	       ok=false;
	   }
	   i++;
	}

	if(ok) {
	    Data.DTWCompareAll( );
		
	    int sel = Data.getOneResult().getTIndex();
	    jList1.setSelectedIndex(sel);
	    TempPlotter.setValues( Data.getTemplate(sel));
		
	    ShowResults( );
	    logButton.enable();
	    logButton.setForeground(java.awt.Color.black);
	}
    } /** compareAllButton_actionPerformed() ends */

    /** Show algorithm run results */
    void ShowResults( ){
        DTWOneResult result = Data.getOneResult();

        int row = jList1.getSelectedIndex();
        if( result.getWarpingPath() != null && result != null ){
            PathPlotter.initializePath( Data.getWindowWidth(),
            Data.getObservation().getValues().length,
            Data.getTemplate(row).getValues().length);
            PathPlotter.setValues( result );
        }
    }

    /** DeleteOne button clicked */
    void deleteOneButton_actionPerformed(ActionEvent e) {
        int row;
        row = jList1.getSelectedIndex();
        if( row != -1 ){
            Data.deleteTemplate(row);
            UpdateList();
        }
    } /** deleteOneButton_actionPerformed() ends */

    /** DeleteAll button clicked */
    void deleteAllButton_actionPerformed(ActionEvent e) {
        Data.deleteAllTemplates();
        UpdateList();
    } /** deleteAllButton_actionPerformed() ends */

    /** Log button clicked */
    void logButton_actionPerformed(ActionEvent e) {
        boolean confirm=true;
	JFileChooser fc = new JFileChooser(new java.io.File("."));
		String Filename;
		
        if( logButton.isEnabled() ){
			if( fc.showSaveDialog( this ) == JFileChooser.APPROVE_OPTION ){
				Filename = fc.getSelectedFile().toString();
				if (fc.getSelectedFile().exists()) {
				    confirm=(JOptionPane.showConfirmDialog(this, 
									  "Are you sure you want to overwrite file "+Filename+"?", 
									  "Confirm overwrite", 
									  JOptionPane.YES_NO_OPTION,
									  JOptionPane.INFORMATION_MESSAGE) == JOptionPane.YES_OPTION);
				}
				if(confirm) {
				    try{
					FileIO.saveLog(Filename, Data);
					
				    }catch( DTWFileIOException fex ){
					// Report error
					JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
				    }
				}
			}
        }
    }/** logButton_actionPerformed() ends */

    /** Load observation menu item selected */
    void jMenuItem2_actionPerformed(ActionEvent e) {
        JFileChooser fc = new JFileChooser(new java.io.File("."));
		String Filename;
		
        fc.setDialogTitle( "Load observation" );
		if( fc.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION ){
    	    Filename = fc.getSelectedFile().toString();
			
			try{
				FileIO.loadObservation(Filename, Data);
                ObsPlotter.setValues( Data.getObservation() );
				
			}catch( DTWFileIOException fex ){
				// Report error
				JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
			}
        }
    }

    /** Load templates menu item selected */
    void jMenuItem3_actionPerformed(ActionEvent e) {
        JFileChooser fc = new JFileChooser(new java.io.File("."));
		String Filename;
		
        fc.setDialogTitle( "Load templates" );
		if( fc.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION ){
			
			Filename = fc.getSelectedFile().toString();
			
			try{
				FileIO.loadTemplates(Filename, Data);
                UpdateList( );
				
			}catch( DTWFileIOException fex ){
				// Report error
				JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
			}
		}
    }
	
    void UpdateList( ){
        int i;
        int count;
        DTWTemplateListItem[] items;
        items = Data.getTemplateList();
        count = (items.length);

        if( count != 0 ){

            String[] data = new String[count];
            for( i=0; i<count; i++ ){
                data[i] = Data.getTemplateName(i);
            }

            jList1.setListData( data );
        }
        else{
	    String[] data = new String[0];
	    jList1.setListData( data );
        }
    }

    void ListSelection( MouseEvent e ){
        if (e.getClickCount() == 1) {
            int index = jList1.locationToIndex(e.getPoint());
            if( index != -1 )
                TempPlotter.setValues( Data.getTemplate(index) );
	    resetAll();
        }
    }

    /** Save observation menu item clicked */
    void jMenuItem4_actionPerformed(ActionEvent e) {
        boolean confirm=true;
	JFileChooser fc = new JFileChooser(new java.io.File("."));
	String Filename;
		
        fc.setDialogTitle( "Save observation" );
		if( fc.showSaveDialog( this ) == JFileChooser.APPROVE_OPTION ){
			
		    Filename = fc.getSelectedFile().toString();
		    if (fc.getSelectedFile().exists()) {
			confirm=(JOptionPane.showConfirmDialog(this, 
							       "Are you sure you want to overwrite file "+Filename+"?", 
							       "Confirm overwrite", 
							       JOptionPane.YES_NO_OPTION,
							       JOptionPane.INFORMATION_MESSAGE) == JOptionPane.YES_OPTION);
		    }
		    if(confirm) {
			try{
			    FileIO.saveObservation(Filename, Data);
			    
			}catch( DTWFileIOException fex ){
			    // Report error
			    JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
			}
		    }
        }
    }
	
    /** Save templates menu item clicked */
    void jMenuItem5_actionPerformed(ActionEvent e) {
        boolean confirm=true;
	JFileChooser fc = new JFileChooser(new java.io.File("."));
	String Filename;
		
        fc.setDialogTitle( "Save templates" );
	if( fc.showSaveDialog( this ) == JFileChooser.APPROVE_OPTION ){
	    Filename = fc.getSelectedFile().toString();
	    if (fc.getSelectedFile().exists()) {
		confirm=(JOptionPane.showConfirmDialog(this, 
						       "Are you sure you want to overwrite file "+Filename+"?", 
						       "Confirm overwrite", 
						       JOptionPane.YES_NO_OPTION,
						       JOptionPane.INFORMATION_MESSAGE) == JOptionPane.YES_OPTION);
	    }
	    if(confirm) {	
		try{
		    FileIO.saveTemplates(Filename, Data);
		}catch( DTWFileIOException fex ){
		    // Report error
		    JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
		    
		}
	    }
        }
    }

    /** Load data set menu item selected */
    void jMenuItem1_actionPerformed(ActionEvent e) {
        JFileChooser fc = new JFileChooser(new java.io.File("."));
        DTWDataVector obs;
		String Filename;
        int numlines;
		
        fc.setDialogTitle( "Open data set" );
		if( fc.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION ){
			Filename = fc.getSelectedFile().toString();
			
			try{
				numlines = FileIO.openDataSet(Filename);
                DTWDataSelector sel = new DTWDataSelector(this, Data, FileIO, numlines);
                sel.show();
                obs = Data.getObservation();
                if( obs != null )
					if( obs.getValues() != null )
						ObsPlotter.setValues(obs);
                UpdateList();
                FileIO.closeDataSet();
				
			}catch( DTWFileIOException fex ){
				// Report error
				JOptionPane.showMessageDialog(this, new String(fex.toString().substring(fex.toString().indexOf(":")+1, fex.toString().length())) );
			}
        }
    }

    /** Warping window width checkbox */
    void jCheckBox1_ItemStateChange(ItemEvent e) {
		if( e.getStateChange() == ItemEvent.SELECTED ){
			jTextField1.enable();
			if( jTextField1.getText() == null ){
				jTextField1.setText( new String( "0" ) );
			}
			parameters.setWwidth( Integer.parseInt( jTextField1.getText() ));
		}
		else{
			jTextField1.disable();
			parameters.setWwidth( DTWParameters.NoWarpingWindow );
		}
		// Set parameters
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    /** Allowable starting and ending point offset */
    void jCheckBox2_ItemStateChange(ItemEvent e) {
		if( e.getStateChange() == ItemEvent.SELECTED ){
			jTextField2.enable();
			if( jTextField2.getText() == null ){
				jTextField2.setText( new String( "0" ) );
			}
            parameters.setSeoffset( Integer.parseInt( jTextField2.getText() ));
		}
		else{
			jTextField2.disable();
            parameters.setSeoffset( 0 );
		}
		
		// Set parameters
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    /** Update warping window width */
    void updatePar1( ){
        int val;
        try{
            val = Integer.parseInt( jTextField1.getText() );
			if( val < 0 )
				throw new NumberFormatException();
            parameters.setWwidth(val);
            prevValue1 = new Integer(val).toString();
        }
        catch( Exception ex ){
            jTextField1.setText( prevValue1 );
        }
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    /** Update allowable starting/ending point offset */
    void updatePar2( ){
        int val;
        try{
            val = Integer.parseInt( jTextField2.getText() );
			if( val < 0 )
				throw new NumberFormatException();
            parameters.setSeoffset(val);
            prevValue2 = new Integer(val).toString();
        }
        catch( Exception ex ){
            jTextField2.setText( prevValue2 );
        }
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    /** Euclidian distance metrics selected */
    void jRadioButton1_itemStateChanged(ItemEvent e) {
		/** Set parameters */
        parameters.setDmetrics( DTWParameters.EuclidianDistanceMetrics );
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    /** Derivative distance metrics selected */
    void jRadioButton2_itemStateChanged(ItemEvent e) {

		/** Set parameters */
        parameters.setDmetrics( DTWParameters.DerivativeDistanceMetrics );
        Data.setDTWParameters( parameters );
		resetAll();
    }
	
    void resetAll(){
        PathPlotter.setValues(null);
        logButton.disable();
        logButton.setForeground(java.awt.Color.gray);
    }
    
}
