package org.dtrend;

import javax.swing.JPanel;
import java.awt.*;

/**
 * <p>Title: DTREND</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class DTWPathPlotter extends JPanel {
    /** Graphics area height */
    int gHeight;
    /** Graphics area height offset from edge of plotting area to panel height */
    int hOffset;
    /** Graphics area width */
    int gWidth;
    /** Graphics area width offset from left to edge of plotting area*/
    int wOffset;
    /** Offset above the plotting graphics for containing the plotting name */
    int topOffset;

    /** Plotting data */
    int[][] values;
    /** Plotting length */
    String DtwDistance = new String();

    /** Maximum x and y values */
    int maxXValue=0, maxYValue=0;

    /** Warping window width */
    int wwidth=Integer.MAX_VALUE;

    /** Is the plotter initialized */
    boolean initialized=false;

    /** Graphics object for drawing */
    java.awt.Graphics graphics;
    /** Double buffer Image */
    java.awt.Image offScreenImage;

    /** Create a new plotter */
    public DTWPathPlotter() {
    } /** DTWPathPlotter() ends */

    /** Class constructor
     * @param width Plotter width
     * @param height Plotter height
     */
    public DTWPathPlotter( int width, int height ) {
		try  {
			jbInit( width, height );
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
    } /** DTWPathPlotter(int, int) ends */
	
    /** Create the buffer image */
    protected void createBufferImage( ){
		offScreenImage = createImage( this.getWidth(), this.getHeight() );
        if( offScreenImage != null ){
            graphics = offScreenImage.getGraphics();
        }
    } /** createBufferImage() ends */
	
    /** Set layout and base data for plotting
     * @param width Plotter width
     * @param height Plotter height
     * @throws Exception any exception caught is thrown to parent.
     */
    private void jbInit( int width, int height ) throws Exception {
		this.setSize( width, height );
        gHeight = this.getHeight()-20;
        gWidth = this.getWidth()-30;
		
        hOffset = this.getHeight()-gHeight;
        wOffset = this.getWidth()-gWidth;
        topOffset = 20;
		
        /** Set component size to width*height */
        this.setMinimumSize( new Dimension( width, height ));
        this.setPreferredSize( new Dimension( width, height ));
        this.setSize( width, height );
    } /** jbInit() ends */
	
    /** Copy values from newValues to classes data structures.
     * @param newValues New values for plotting
     */
    public void setValues( DTWOneResult result ){
        if( result != null ){
            int[][] newValues = result.getWarpingPath();
            DtwDistance = new Float(result.getDtwDistance()).toString();
			
            if( newValues == null ){
                values = null;
                drawPlotting();
				DtwDistance = "";
            }else{
                values = new int[ newValues.length ][ newValues[0].length];
                System.arraycopy( newValues, 0, values, 0, newValues.length );
				
				if((newValues[newValues.length-1][0] > maxXValue) || (newValues[newValues.length-1][1] > maxYValue)) {
					maxXValue=newValues[newValues.length-1][0];
					maxYValue=newValues[newValues.length-1][1];
				}
				
				if(initialized) drawPlotting();
				initialized=false;
            }
        }
		else{
			values = null;
			drawPlotting();
			DtwDistance = "";
		}
    } /** setValues() ends */
	
    /** Initialize the component for drawing the warping path.
     *  Must be called always prior to calling setValues!
     * @param windowWidth Warping window width
     * @param n Observation length
     * @param m Template length
     */
    public void initializePath(int windowWidth, int n, int m) {
		values=null;
		wwidth=windowWidth;
		maxXValue=n-1;
		maxYValue=m-1;
		initialized=true;
    }
	
    /** API call to draw the image. */
    protected void drawPlotting( ){
        /**
         * Draw plotting by calling paint method.
         */
		paint( this.getGraphics() );
    }
	
    /** Update graphics in component
     * @param g Graphics object to draw into
     */
    public void update( Graphics g ){
		paint( g );
    }
	
    private int minint(int a, int b){
		return(a<b?a:b);
    }
	
    private int maxint(int a,int b){
		return(a>b?a:b);
    }
	
    /** Paint the component
     * @param g Graphics object to draw into
     */
    public void paint( Graphics g ){
        int i;
        int[][] plotValues;
		
        float xDistance;
        float yDistance;
		
        int currentX;
        int currentY;
        int nextX;
        int nextY;
        Font font;
		
        /** Create double buffer. */
		createBufferImage();
		
        if( offScreenImage != null ){
            /** Draw a white filled rectangle surrounded by a black one for the plotting */
    	    graphics.setColor( java.awt.Color.white );
			graphics.fillRect( wOffset, topOffset, gWidth-2, gHeight-topOffset );
			graphics.setColor( java.awt.Color.black );
			graphics.drawRect( wOffset, topOffset, gWidth-2, gHeight-topOffset );
			
            /** Check if there's values in the array */
            if( values != null ){
				
                /** Apply factor based on the plotter height on plotValues */
				
                xDistance = ((float)gWidth-2f) / (float)maxXValue;
                yDistance = ((float)gHeight-(float)topOffset) / (float)maxYValue;
				
				plotValues = new int[values.length][2];
				for( i=0; i<values.length; i++ ){
                    plotValues[i][0] = new Float( values[i][0] * xDistance).intValue();
                    plotValues[i][1] = new Float( values[i][1] * yDistance).intValue();
				}
				
				if((wwidth!=Integer.MAX_VALUE)&&(wwidth>=Math.abs(maxYValue-maxXValue))) {
					/** Draw warping window */
					currentX=wOffset;
					currentY=gHeight-(int)(wwidth*yDistance);
					nextX=wOffset+(int)((maxYValue-wwidth)*xDistance);
					nextY=topOffset;
					
					graphics.setColor(java.awt.Color.blue);
					graphics.drawLine(currentX, currentY, nextX, nextY);
					
					currentY=gHeight;
					currentX=wOffset+(int)(wwidth*xDistance);
					nextX=wOffset+gWidth-2;
					nextY=gHeight-(int)((maxXValue-wwidth)*yDistance);
					
					graphics.drawLine(currentX, currentY, nextX, nextY);
				}
				
				/** Adjust distances for drawing the grid*/
				while( xDistance < 10 ) {
                    xDistance *= 2;
                }
                while( yDistance < 10 ) {
                    yDistance *= 2;
                }
				
    	        graphics.setColor( java.awt.Color.lightGray );
                nextX=(int)xDistance + wOffset;
                i=1;
                do{
                    graphics.drawLine(nextX, gHeight, nextX, topOffset);
                    i++;
                    nextX = (int)(wOffset + xDistance*i);
                } while( nextX < (wOffset + gWidth-2) );
				
                nextY=(int)(gHeight-yDistance+1);
                i=1;
                do{
                    graphics.drawLine(wOffset, nextY, gWidth+wOffset, nextY);
                    i++;
                    nextY = (int)(gHeight-yDistance*i+1);
                } while( nextY > topOffset );
				
                graphics.setColor( java.awt.Color.black );
				
                /** Set plotting start points and distance between two points on the x axel. */
				currentX = wOffset + plotValues[0][0];
				currentY = gHeight - plotValues[0][1];
				
                graphics.fillOval(currentX-3,currentY-3,6,6);
				
				/** Draw the plotting */
				for( i=1; i<values.length; i++ ){
                    nextX = (wOffset + plotValues[i][0]);
                    nextY = (gHeight) - plotValues[i][1];
                    graphics.drawLine(currentX, currentY, nextX, nextY);
					
                    /** Set the new coordinates. */
                    currentX = nextX;
                    currentY = nextY;
				}
				
				graphics.fillOval(currentX-3,currentY-3,6,6);
			}
			
            /** Draw labels */
            font = graphics.getFont().deriveFont(10);
            graphics.setFont( font );
            graphics.drawString( "DTW distance: " + DtwDistance, gWidth/2, 12 );
			
            /** Copy double buffer to panel */
			g.drawImage( offScreenImage, 0, 0, this.getWidth(), this.getHeight(), this );
        }
    } /** paint() ends */
}
