/*
DTWSelectionPlotter.java -- DTREND PROJECT
Author: Tero Rontti
Version: 0.1
Revision date: 23.2.2002
Creation date: 10.2.2002


DTREND PROJECT
Project manager: Juha Ylipekkala
Project group: Timo Ahonen, Tero Rontti, Juha Ylipekkala

-------------------------------------------------------------------------------
REVISION HISTORY

Version 0.00
Initial version, not working.
Author: Tero Rontti
Date: 23.2.2002
Known bugs/problems:
- Does not work as specified to.
- Lack of comments in code
*/

package org.dtrend;

import java.awt.*;

/** This class defines the Selection plotter for DTREND tool
 * @author foo
 * @version 0.0
 */
public class DTWSelectionPlotter extends DTWHorizontalPlotter {
    int selectionStart;
    int selectionEnd;

    public DTWSelectionPlotter( int width, int height ) {
        super(width, height);
        selectionStart = 0;
        selectionEnd = 0;
    }
	
    /** Init layout */
    private void jbInit() throws Exception {
    } /** jbInit() ends */
	
    /** Set selection
     * @param selStart Selection start point
     * @param selEnd Selection end point
     */
    public void setSelection( int selStart, int selEnd ){
        selectionStart = selStart;
        selectionEnd = selEnd;
    }
	
    /** Paint the component
     * @param g Graphics object to draw into
     */
    public void paint( Graphics g ){
        float maxValue = 0f;
        float minValue = 0f;
        int minIndex = -1;
        int i;
        int[] plotValues;
        float plotDistance;
        int currentX;
        int currentY;
        int nextX;
        int nextY;
        Font font;

        /** Create double buffer. */
		createBufferImage();

        if( offScreenImage != null ){

            /** Draw a white filled rectangle surrounded by a black one for the plotting */
    	    graphics.setColor( java.awt.Color.white );
			graphics.fillRect( wOffset, 1, gWidth-2, gHeight);
			graphics.setColor( java.awt.Color.black );
            graphics.drawRect( wOffset, 1, gWidth-2, gHeight );
			
            /** Check if there's values in the array */
            if( values != null ){
				
                /** Find the maximum value. */
				for( i=0; i<values.length; i++){
                    if( values[i] > maxValue )
						maxValue = values[i];
                    if( values[i] < minValue ){
                        minValue = values[i];
                        minIndex = i;
                    }
				}
				
                /** Set MaxValue for the label */
                MaxValue = Integer.toString( new Float( maxValue ).intValue() );
                MinValue = Integer.toString( new Float( minValue ).intValue() );
				
                /** Set axis level */
                int axis = 0;
                if( minValue != 0 ){
                    axis = (int)(((gHeight-1) / (minValue-maxValue) ) *(minValue) );
                }
				
                /** Apply factor based on the plotter height on plotValues */
				plotValues = new int[values.length];
				for( i=0; i<values.length; i++ ){
                    plotValues[i] = new Float( values[i] * (( gHeight-1 )  / (-minValue+maxValue)) +axis ).intValue();
				}
				
                /** Set plotting start points and distance between two points on the x axel. */
				plotDistance = (((float)gWidth-2f) / ((float)values.length-1f));
				currentX = (wOffset + 2);
				currentY = (gHeight - plotValues[0]);
				
                if( minIndex != -1 ){
                    graphics.setColor( java.awt.Color.lightGray );
                    graphics.drawLine( wOffset+2, gHeight-axis, wOffset+gWidth-2, gHeight-axis );
                    graphics.setColor( java.awt.Color.black );
                }
				
                /** Draw the plotting */
				for( i=1; i<values.length; i++ ){
                    nextX = (wOffset + (int)(plotDistance * i));
                    nextY = (gHeight) - plotValues[i];
                    if( selectionStart < i ){
                        if( selectionEnd+1 > i ){
                            graphics.setColor( java.awt.Color.black );
                            graphics.fillRect(currentX, 2, nextX-currentX, gHeight-2 );
                            graphics.setColor( java.awt.Color.white );
                            graphics.drawLine(currentX, currentY, nextX, nextY);
                        }
                        else{
                            graphics.setColor( java.awt.Color.black );
                            graphics.drawLine(currentX, currentY, nextX, nextY);
                        }
                    }else{
                        graphics.drawLine(currentX, currentY, nextX, nextY);
                    }
					
                    /** Set the new coordinates. */
                    currentX = nextX;
                    currentY = nextY;
				}
				
                if( selectionStart == selectionEnd ){
                    graphics.setColor( java.awt.Color.black );
                    nextX = (wOffset + (int)(plotDistance * selectionStart ));
                    nextY = (gHeight) - plotValues[selectionStart];
                    graphics.drawLine( nextX, 2, nextX, gHeight );
                    graphics.setColor( java.awt.Color.white );
                    graphics.drawLine( nextX, nextY, nextX, nextY );
                }
				
			}
			
            /** Draw labels */
            graphics.setColor( java.awt.Color.black );
            font = graphics.getFont().deriveFont(10);
            graphics.setFont( font );
            graphics.drawString( Name, wOffset, gHeight+12 );
            graphics.drawString( Length, gWidth, gHeight+12 );
            graphics.drawString( MaxValue, 0, 12  );
            graphics.drawString( MinValue, 0, gHeight );

            /** Copy double buffer to panel */
			g.drawImage( offScreenImage, 0, 0, this.getWidth(), this.getHeight(), this );
        }
    } /** paint() ends */
	
}
