/*
DTWVerticalPlotter.java -- DTREND PROJECT
Author: Tero Rontti
Version: 0.1
Revision date: 23.2.2002
Creation date: 10.2.2002


DTREND PROJECT
Project manager: Juha Ylipekkala
Project group: Timo Ahonen, Tero Rontti, Juha Ylipekkala

-------------------------------------------------------------------------------
REVISION HISTORY

Version 0.1
Initial working version
Author: Tero Rontti
Date: 23.2.2002
Known bugs/problems:
*/

package org.dtrend;

import java.awt.*;
import javax.swing.*;

/** This class defines the Vertical plotter component used by the UI
 * @author Tero Rontti
 * @version 0.1
 */
public class DTWVerticalPlotter extends DTWDataPlotter {
    /** Offset above the plotting graphics for containing the plotting name */
    int topOffset;

    /** Creates a new data plotter
     * @param width Plotter width
     * @param height Plotter height
     */
    public DTWVerticalPlotter(int width, int height ) {

	/** Construct super() with the same dimensions. */
        super( width, height );

        topOffset = 20;
    }

    /** Paint the component
     * @param g Graphics object to paint into
     */
    public void paint( Graphics g ){
        float maxValue = 0f;
        float minValue = 0f;
        int minIndex = -1;
        int i;
	/** Scaled plotting values */
        int[] plotValues;
        float plotDistance;
        int currentX;
        int currentY;
        int nextX;
        int nextY;
        Font font;

        /** Create double buffer. */
		createBufferImage( );

        /** Draw the plotting area */
		graphics.setColor( java.awt.Color.white );
		graphics.fillRect( wOffset, topOffset, gWidth-1, (gHeight-1-topOffset) );
		graphics.setColor( java.awt.Color.black );
		graphics.drawRect( wOffset, topOffset, gWidth-1, (gHeight-1-topOffset) );
		
        /** Check if there are any values. */
        if( values != null ){
			
            // Find the maximum value
			for( i=0; i<values.length; i++){
                if( values[i] > maxValue )
					maxValue = values[i];
                if( values[i] < minValue ){
                    minValue = values[i];
                    minIndex = i;
                }
			}
			
            MaxValue = Integer.toString( new Float( maxValue ).intValue() );
            MinValue = Integer.toString( new Float( minValue ).intValue() );
			
            /** Set axis level */
            int axis = 0;
            if( minValue != 0 ){
                axis = (int)(((gWidth-1) / (minValue-maxValue) ) *minValue );
            }
			
            /** Apply factor based on maximum value and the plotter width to plotValues */
			plotValues = new int[values.length];
			for( i=0; i<values.length; i++ ){
                plotValues[i] = new Float( values[i] * ( gWidth - 1 ) / (maxValue-minValue) + axis ).intValue();
			}
			
            /** Set distance between two points on the y axis */
			plotDistance = (((float)gHeight - (float)topOffset) / ((float)values.length - 1f));
			
            if( minIndex != -1 ){
                graphics.setColor( java.awt.Color.lightGray );
                graphics.drawLine( wOffset+gWidth-axis, topOffset+gHeight-hOffset, wOffset+gWidth-axis, topOffset+1 );
                graphics.setColor( java.awt.Color.black );
            }
			
            /** Set start points for plotting. */
			currentX = wOffset + gWidth-2-plotValues[0];
			currentY = gHeight-2;
			
			/** Draw lines connecting plotting points */
			for( i=1; i<values.length; i++ ){
				nextX = wOffset + gWidth - plotValues[i];
				nextY = gHeight - (int)(plotDistance * i );
				
				graphics.drawLine(currentX, currentY, nextX, nextY);
				
				/** Set new coordinates for current point */
				currentX = nextX;
				currentY = nextY;
			}
		}
		
        /** Draw labels */
        font = graphics.getFont().deriveFont(10);
        graphics.setFont( font );
        graphics.drawString( Name, wOffset, 12 );
        graphics.drawString( Length, 0, topOffset+12 );
        graphics.drawString( MaxValue, wOffset, gHeight+12  );
        graphics.drawString( MinValue, gWidth, gHeight+12 );
		
        /** Copy double buffer to screen */
		g.drawImage( offScreenImage, 0, 0, this.getWidth(), this.getHeight(), this );
		
    }
}
